import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)

/**
 * A rocket that can be controlled by the arrowkeys: up, left, right.
 * The gun is fired by hitting the 'space' key. 'z' releases a proton wave.
 * 
 * @author Poul Henriksen
 * @author Michael Kolling
 * 
 * @version 1.0
 */
public class Rocket extends SmoothMover
{
    private static final int gunReloadTime = 5;         // The minimum delay between firing the gun.
    private static final int protonReloadTime = 100;     

    private int reloadDelayCount;               // How long ago we fired the gun the last time.
    private boolean flames = false;
    private int protonReloadCount = 0;

    private GreenfootImage rocket = new GreenfootImage("rocket.png");    
    private GreenfootImage rocketWithThrust = new GreenfootImage("rocketWithThrust.png");

    private int shieldCount = 0;
    private int shieldMin = 100;
    private int shieldMax = 250;
    private boolean shieldOn = false;
    private boolean everyOther = false;
    private String imagePrefix = ""; 

    private ShieldCounter shieldC;

    /**
     * Initilise this rocket.
     */
    public Rocket()
    {

        reloadDelayCount = 5;
        Vector v = new Vector(0,.3);
        addForce(v);

    }
    /**
     * Do what a rocket's gotta do. (Which is: mostly flying about, and turning,
     * accelerating and shooting when the right keys are pressed.)
     */
    public void act()
    {
        checkKeys();
        reloadDelayCount++;
        protonReloadCount++;

        if(shieldOn)
        {
            shieldCount = shieldCount - 2;
            if(shieldCount>0)
            {
                Space s = (Space)getWorld();
                s.resetShieldMeter(shieldCount);
            }

            if(shieldOn && shieldCount<0)
            {
                shieldCount = 0;
                shieldOn = false;
                imagePrefix = "";
                if(!flames)
                    this.setImage(imagePrefix + "rocket.png");
                else
                    this.setImage(imagePrefix + "rocketWithThrust.png");
            }
        }
        else if(shieldCount <=shieldMax)
        {
            if(everyOther)
            {
                shieldCount++;
                Space s = (Space)getWorld();
                s.resetShieldMeter(shieldCount);
            } 
            everyOther = !everyOther;
        }
        move();

        if(protonReloadCount <= protonReloadTime)
        {
            Space s = (Space) getWorld();
            s.updateTimer(protonReloadTime,protonReloadCount);
        }
        checkCollision();

    }

    /**
     * Check whether there are any key pressed and react to them.
     */
    private void checkKeys() 
    {
        if (Greenfoot.isKeyDown("space")) 
        {
            fire();
        }
        if (Greenfoot.isKeyDown("up")) 
        {
            ignite(true);
        }
        else
        {
            ignite(false);
        }
        if (Greenfoot.isKeyDown("down")) 
        {

        }
        if (Greenfoot.isKeyDown("right")) 
        {
            turn(5);
        }
        if (Greenfoot.isKeyDown("left")) 
        {
            turn(-5);
        }
        if(Greenfoot.isKeyDown("e"))
        {
            if(protonReloadCount>protonReloadTime)
            {
                startProton();
                Space s = (Space) getWorld();
                s.protonWaveUsed();
            }   
        }
        if (Greenfoot.isKeyDown("f")) 
        {
            if(shieldCount>=shieldMin && !shieldOn)
            {
                shieldOn = true;
                imagePrefix = "shielded";
                if(flames)
                    this.setImage(imagePrefix + "rocketWithThrust.png");
                else
                    this.setImage(imagePrefix + "rocket.png");
            }
        }
    }

    /**
     * Fire a bullet if the gun is ready.
     */
    private void fire() 
    {
        if (reloadDelayCount >= gunReloadTime) 
        {
            Bullet bullet = new Bullet (getMovement().copy(), getRotation());
            getWorld().addObject (bullet, getX(), getY());
            bullet.move ();
            reloadDelayCount = 0;
        }
    }

    /**
     * Changes picture of rocket and adds force
     */
    private void ignite(boolean isThursting)
    {
        if(isThursting)
        {
            Vector v = new Vector(getRotation(),.1);
            addForce(v);
            if(!flames)
            {
                this.setImage(imagePrefix + "rocketWithThrust.png");
                flames = true;
            }
        }
        else
        {
            if(flames)
            {
                this.setImage(imagePrefix + "rocket.png");
                flames = false;
            }

        }
    }

    /**
     * Checks for collisions with astroids
     */
    private void checkCollision()
    {
        Actor a = getOneIntersectingObject(Asteroid.class);
        if(a != null && !shieldOn)
        {
            Explosion e = new Explosion();
            //getWorld().gameOver();
            getWorld().addObject(e,this.getX(),this.getY());
            Space space = (Space) getWorld();
            space.gameOver();
            getWorld().removeObject(this);

        }
    }

    /**
     * Creates a proton wave that goes out from the spaceship
     */
    private void startProton()
    {
        protonReloadCount = 0;
        ProtonWave wave = new ProtonWave();
        getWorld().addObject(wave,getX(),getY());
    }

    public void resetLocation()
    {
        setLocation(getWorld().getWidth()/2 + 100, getWorld().getHeight()/2);
    }
}

